function b = feasiblePostLassoMatTool(y,x,varargin)

[n,p] = size(x);

[MaxIter,UpsTol,lambda,beta0,clusterVar,dist,link,glmTol,initScale] = process_options(varargin,...
    'MaxIter',15,'UpsTol',1e-6,'lambda',2.2*sqrt(n)*norminv(1-(.1/log(n))/(2*p)),...
    'beta0',[],'clusterVar',[],'Dist','normal','linkFun','identity','glmTol',1e-8,'initScale',.5);

lambda = lambda/(2*n);

if isempty(clusterVar)
    if isempty(beta0)
        if strcmp(link,'logit')
            Syx = x.*((y-mean(y))*ones(1,p));
        else
            Syx = x.*((y-mean(y))*ones(1,p));
        end
        Ups0 = std(Syx);
        b = lassoglm(x./(ones(n,1)*Ups0),y,dist,'link',link,'Lambda',initScale*lambda,'Standardize',0,'RelTol',glmTol);
        use = abs(b) > 0;
        if strcmp(link,'logit')
            e = y-glmval(glmfit(x(:,use),y,'binomial','link','logit'),x(:,use),'logit');
        else
            e = y-x(:,use)*(x(:,use)\y);
        end
    else
        if strcmp(link,'logit')
            e = y - 1./(1+exp(-x*beta0));
        else
            e = y - x*beta0;
        end
        Syx = x.*(e*ones(1,p));
        Ups0 = std(Syx);
        b = lassoglm(x./(ones(n,1)*Ups0),y,dist,'link',link,'Lambda',lambda,'Standardize',0,'RelTol',glmTol);
        use = abs(b) > 0;
        if strcmp(link,'logit')
            e = y-glmval(glmfit(x(:,use),y,'binomial','link','logit'),x(:,use),'logit');            
        else
            e = y-x(:,use)*(x(:,use)\y);
        end            
    end
    
    kk = 1;
    Syx = x.*(e*ones(1,p));
    Ups1 = std(Syx);
    while norm(Ups0-Ups1) > UpsTol && kk < MaxIter,
        d0 = norm(Ups0-Ups1);
        b = lassoglm(x./(ones(n,1)*Ups1),y,dist,'link',link,'Lambda',lambda,'Standardize',0,'RelTol',glmTol);
        use = abs(b) > 0;
        if sum(use) < size(x,2)-1
            if strcmp(link,'logit')
                e = y-glmval(glmfit(x(:,use),y,'binomial','link','logit'),x(:,use),'logit');
            else
                e = y-x(:,use)*(x(:,use)\y);
            end
            Ups0 = Ups1;
            Syx = x.*(e*ones(1,p));
            Ups1 = std(Syx);
            kk = kk+1;
            d1 = norm(Ups0 - Ups1);
            if d1 == d0
                Ups0 = Ups1;
            end
        else
            kk = MaxIter + 1;
        end
    end
    b = b./Ups1';
else
    Dt = dummyvar(clusterVar);
    if isempty(beta0)
        if strcmp(link,'logit')
            Syx = x.*((y-mean(y))*ones(1,p));
        else
            Syx = x.*((y-mean(y))*ones(1,p));
        end
        Ups0 = std(Dt'*Syx)';
        b = lassoglm(x./(ones(n,1)*Ups0),y,dist,'link',link,'Lambda',initScale*lambda,'Standardize',0,'RelTol',glmTol);
        use = abs(b) > 0;
        if strcmp(link,'logit')
            e = y-glmval(glmfit(x(:,use),y,'binomial','link','logit'),x(:,use),'logit');            
        else
            e = y-x(:,use)*(x(:,use)\y);
        end            
    else
        if strcmp(link,'logit')
            e = y - 1./(1+exp(-x*beta0));
        else
            e = y - x*beta0;
        end
        Syx = x.*(e*ones(1,p));
        Ups0 = std(Dt'*Syx)';
        b = lassoglm(x./(ones(n,1)*Ups0),y,dist,'link',link,'Lambda',lambda,'Standardize',0,'RelTol',glmTol);
        use = abs(b) > 0;
        if strcmp(link,'logit')
            e = y-glmval(glmfit(x(:,use),y,'binomial','link','logit'),x(:,use),'logit');            
        else
            e = y-x(:,use)*(x(:,use)\y);
        end            
    end
    
    kk = 1;
    Syx = x.*(e*ones(1,p));
    Ups1 = std(Dt'*Syx)';
    while norm(Ups0-Ups1) > UpsTol && kk < MaxIter,
        d0 = norm(Ups0-Ups1);
        b = lassoglm(x./(ones(n,1)*Ups1),y,dist,'link',link,'Lambda',lambda,'Standardize',0,'RelTol',glmTol);
        use = abs(b) > 0;
        if sum(use) < size(x,2)-1
            if strcmp(link,'logit')
                e = y-glmval(glmfit(x(:,use),y,'binomial','link','logit'),x(:,use),'logit');
            else
                e = y-x(:,use)*(x(:,use)\y);
            end
            Ups0 = Ups1;
            Syx = x.*(e*ones(1,p));
            Ups1 = std(Dt'*Syx)';
            kk = kk+1;
            d1 = norm(Ups0 - Ups1);
            if d1 == d0
                Ups0 = Ups1;
            end
        else
            kk = MaxIter + 1;
        end
    end
    b = b./Ups1';

end
